#ifndef MAVIS_GST_H
#define MAVIS_GST_H

#include "Mavis_Gst_global.h"
#include <gst/gst.h>
#include <gst/video/videooverlay.h>
#include <QWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QString>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <linux/videodev2.h>
#include <sys/ioctl.h>
#include <QDebug>
#include <QtDBus/QDBusConnection>
#include <QtDBus/QDBusConnectionInterface>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusReply>
#include <QSensor>
#include <QOrientationSensor>
//#include "motor_client.h"
#include <QTimer>
#include "_freecplus.h"
#include <gstcamera3ainterface.h>
#include "mavis_motor.h"
#include "pngthread.h"
#include "GlobalSignal.h"
#include <QPainter>
#include "videodisplay.h"

//dbus接口相关
struct dbus_set_para
{
    char *server_name;
    char *object_path;
    char *interface;
    char *method;
};

//曝光模式（自动曝光，手动曝光）
typedef enum expose_mode{
   ae_mode_manual = 1,   //手动曝光
   ae_mode_auto          //自动曝光
}ExposeMode;
//对焦模式（自动对焦，手动对焦）
typedef enum af_mode{
   gst_camerasrc_af_mode_off = 1,          //手动对焦
   gst_camerasrc_af_mode_auto,
   gst_camerasrc_af_mode_continuous_video  //自动对焦
}AfMode;

//自动旋转模式是否开启
typedef enum autorotation_mode{
    on_autorotation = 1,
    off_autorotation = 0
}AutoRotationMode;
//pc/平板模式
typedef enum pc_pad_mode{
    on_pcmode = 0,
    on_padmode = 1
}PcPadMode;
typedef enum rotationmode
{
    normal_s=1,      //水平向下，不改变pc/table模式，不改变前一个信号值
    upsidedown_s=2,  //
    left_s=3,        //
    right_s=4 ,      //
    no_rotation = 0,    //
}RotationMode;
//相机旋转角度
typedef enum camera_roll_mode{
    REAR_MODE = 0,
    FRONT_MODE = 170,
    SCAN_MODE = 235
}CameraRollMode;
//镜像模式
typedef enum flip_mode{
    noflip = 0,        //无旋转
    cwflip_90 = 1,     //顺时针旋转90
    cwflip_180 = 2,    //顺时针旋转180
    ccwflip_90 = 3,    //逆时针旋转90
    lrflip = 4,        //左右镜像
    udflip = 5,        //上下镜像
    lr_ccwflip_90 = 6, //左右镜像再逆时针旋转90
    lr_cwflip_90 = 7   //左右镜像再顺时针旋转90
}FlipMode;
//镜像模式
typedef enum mirroring_mode{
    NOmirroring = 0,     //无镜像
    LRmirroring = 1,     //左右镜像
    UDmirroring = 2,     //上下镜像
    LRUPmirroring = 3    //上下左右镜像
}MirroringMode;
//操作状态返回码
typedef enum ret_status{
    OPERATOR_OK = 0,
    OPERATOR_ERR = -1,    
    PAR_ERR = -2,        //传参错误
    NO_CREAT = -3,       //设备没有creat
    NO_OPEN = -4,        //设备没有打开
    DEV_BUSY = -5        //硬件被占用
}RetStatus;

typedef enum image_format{
    JPG = 1,
    BMP,
    PNG
}ImageFormat;

typedef enum video_format{
    AVI=1,
    MP4
}VideoFormat;

typedef struct Camera_info
{
    QString image_path;        //图片存储路径
    QString video_path;        //视频存储路径
    QString imageformat;       //图片格式
    QString videoformat;       //视频格式
    QString videoresolution;   //视频分辨率
    QString oldvideoresolution;       //old视频分辨率
    QString previewresolution;        //预览分辨率
    QString oldpreviewresolution;     //old预览分辨率
    QString imageresolution;    //拍照分辨率
    QString oldimageresolution; //old拍照分辨率
    long long pre_time_total;   //记录暂停时，前面录像的总时间
    long long start_time;       //录像当前开始时刻
    bool pause_on;              //是否处于暂停状态
    int motorAngle;             //当前马达角度
    int oldmotorAngle;
    CameraRollMode now_motormode;   //当前马达前后置状态
    CameraRollMode old_motormode;   //前一次马达前后置状态
    AutoRotationMode now_autorotationmode; //当前自动旋转打开/关闭模式
    PcPadMode now_pcpadmode;               //当前处于pc或pad模式
    RotationMode now_rotationmode;         //当前旋转状态
    RotationMode jinru_rotationmode;
    bool true_false;                       //当前手动镜像值

}Camera_Info;

typedef struct _CustomData {
  GstElement *pipeline;
  GstElement *vsrc,*capsfilter,*aspect_ratiocrop, *tee,*play_queue, *video_flip,*myconvert, *playsink ,*playapp_sink;
  GstCaps *filter_caps,*playappsink_caps;
  GstElement *record_queue, *record_crop,*record_crop_filter,*record_video_rate, *record_video_convert,*record_video_scale,*record_video_capsfilter,*record_x264_enc,*record_video_queue;  //视频文件存储相关
  GstElement *record_avi_mux, *record_avifile_sink;                    //avi混流相关
  GstElement *record_queue_mp4, *record_capsfilter,*record_video_rate_mp4, *record_mp4_videoflip,*record_video_convert_mp4,*record_video_scale_mp4,*record_video_mp4_capsfilter,*record_x264_enc_mp4,*record_video_queue_mp4;  //视频文件存储相关
  GstElement *record_mp4_mux, *record_mp4file_sink;                   //mp4混流相关
  GstPad *tee_record_pad,*queue_record_pad;                           //avi支路pad
  GstCaps *record_video_capsfilter_caps;                              //avi支路caps
  GstPad *record_blockpad,*tee_record_pad_mp4,*queue_record_pad_mp4;                   //MP4支路pad
  GstCaps *record_filter_caps,*record_video_mp4_capsfilter_caps;                          //MP4支路caps
  GstPad *tee_record_pad_appsink,*queue_record_pad_appsink;                   //appsink支路pad

  //zoom in/out 相关
  GstElement *crop_queue,*crop,*crop_filter;
  GstPad *blockpad;

  GstElement *alsa_src , *audio_rate,*audio_convert, *lamemp3_enc, *audio_queue;   //音频相关

  //单独pipeline_x测试，用于测试视频录制暂停的功能
  GstElement *pipeline_x,*vsrc_x,*record_video_convert_x, *record_video_rate_x, *record_queue_x1,*videoscale_x,*record_x264_enc_x,*record_avi_mux_x, *record_avifile_sink_x;  //视频文件存储相关
  GstElement *capsfilter_x,*capsfilter_x1,*capsfilter_x2;
  GstCaps *filter_caps_x,*filter_caps_x1,*filter_caps_x2;

  GstElement *appsink_queue,*appsink_capsfilter,*appsink_crop,*appsink_crop_filter,*appsink_video_scale,*appsink_video_capsfilter,*appsink_videoflip,*appsink_convert,*app_sink;   //appsink 支路
  GstCaps *appsink_filter_caps,*appsink_video_capsfilter_caps,*appsink_caps;
  GstPad *appsink_blockpad;

  // 截图和录像标志
  bool capture_flag = false;
  bool record_flag = false;

  //GMainLoop *main_loop;  /* GLib's Main Loop */
} CustomData;

typedef struct Device_Info{
    QString dev_name;                      //设备名，由调用传入
    QString dev_path;                      //设备路径，暂时为空
    QList<QString> dev_format;             //设备支持的编码格式（YUY2，RGB等）
    QList<image_format> list_imageformat;  //图片格式列表
    QList<video_format> list_videoformat;  //视频格式列表
    QList<QPair<int,int>> list_scale;      //分辨率列表
    QList<QPair<int,int>> list_imagescale; //图片分辨率列表
    QList<int>  list_fps;                  //帧率列表
}DeviceInfo;

typedef struct Kylin_CameraInfo
{
    QString devname;          //系统设备名
    QString imagesavepath;    //图片保存路径
    QString videosavepath;    //视频保存路径
    ImageFormat imageformat;  //图片编码格式
    VideoFormat videoformat;  //视频编码格式
    unsigned int width;       //分辨率-宽
    unsigned int height;      //分辨率-高
    unsigned int imagewidth;  //图片分辨率-宽
    unsigned int imageheight; //图片分辨率-高
    unsigned int fps;         //帧率
}KylinCameraInfo;
class Mavis_Motor;
class MAVIS_GST_EXPORT Mavis_Gst: public QObject
{
    Q_OBJECT
public:
    Mavis_Gst();
    QWidget *videoWindow;

    static Camera_Info camera_info;
    static CustomData date;
    //获取
    RetStatus enum_device(DeviceInfo *device);
    //创建（使用开始前必需调用）
    RetStatus create(QWidget *parent,const KylinCameraInfo device);
    //释放（使用结束后必需调用）
    RetStatus release();
    //3设置摄像头基础参数 需要打开摄像头之前设置
    RetStatus camera_set_param(const KylinCameraInfo *device);
    //4获取摄像头基础参数
    RetStatus camera_get_param(KylinCameraInfo &device);
    //转换成拍照模式
    RetStatus change_to_image();
    //转换成录像模式
    RetStatus change_to_video();
    //打开摄像头
    RetStatus open_camera();
    //关闭摄像头
    RetStatus close_camera();
    //5拍照
    RetStatus camera_take_photo(QString photo_name);
    //录像前抓拍
    RetStatus camera_image_before_record(QString image_name);
    //6开始录像
    RetStatus camera_start_record(QString video_name);
    //7结束录像
    RetStatus camera_stop_record(void);
    //动态解除appsink支路的链接
    RetStatus stop_image_pipe();
    //添加硬件4096P功能，非动态
    RetStatus start_image_pipe_4096P();
    //动态添加appsink支路的链接
    RetStatus start_image_pipe();
    //动态添加录像支路的链接
    RetStatus statrt_record_pipe(QString video_name);
    //9获取当前录制视频的录像时长 单位ms
    long long camera_get_record_time();
    //10设置摄像头细节参数
    int camera_set_control(int id,int value);
    //11获取摄像头细节参数
    int camera_get_control(int id,int value);
    //29获取摄像头支持调节的分辨率列表（预览/录像）
    bool camera_scale_get_list(QList<QPair<int,int>> &list);
    //30获取当前使用摄像头的分辨率（预览/录像）
    bool camera_scale_get(QPair<int,int> &value);
    //33设置当前使用摄像头的分辨率（预览/录像）
    bool camera_scale_set(const QPair<int,int> value);
    //获取摄像头支持调节的分辨率列表（拍照）
    bool camera_imagescale_get_list(QList<QPair<int,int>> &list);
    //获取当前使用的摄像头的分辨率（拍照）
    bool camera_imagescale_get(QPair<int,int> &value);
    //设置当前摄像头拍照的分辨率（拍照）
    bool camera_imagescale_set(const QPair<int,int> value);
    //40获取摄像头支持调节的帧率列表
    bool camera_frameRate_get_list(QList<int> &list);
    //41获取当前使用摄像头的帧率
    bool camera_frameRate_get(int &value);
    //42设置当前使用摄像头的帧率
    bool camera_frameRate_set(const int value);
    //43获取摄像头支持调节的照片格式列表
    bool camera_photoFormat_get_list(QList<ImageFormat> &list);
    //44获取当前摄像头支持调节的照片格式
    bool camera_photoFormat_get(ImageFormat &value);
    //45设置摄像头支持调节的照片格式
    bool camera_photoFormat_set(const ImageFormat value);
    //46获取摄像头支持调节的视频格式
    bool camera_videoFormat_get_list(QList<VideoFormat> &list);
    //47获取当前摄像头支持调节的视频格式
    bool camera_videoFormat_get(VideoFormat &value);
    //48设置当前摄像头支持调节的视频格式
    bool camera_videoFormat_set(const VideoFormat value);
    //设置当前图片保存路径
    bool camera_photoPath_set(const QString value);
    //设置当前视频保存路径
    bool camera_videoPath_set(const QString value);
    //设置是否手动镜像,初始化阶段（没有open_camera之前），调用
    bool camera_mirroring_set_no_open(bool truefalse);
    //设置是否手动镜像，（open_camera之后）调用
    bool camera_mirroring_set(bool truefalse);
    //设置摄像头前后置
    bool camera_motor_rear_front_set(bool angle);
    //应用要求。打开APP就需要设置
    bool camera_motor_rear_front_set_fater_open();
    //设置zoom in/out,zoom_num >= 0 && zoom_num <=25
    void set_crop(int zoom_num);
    //查看拍照是否完成
    RetStatus camera_take_photo_status();     
    //自动旋转开启/关闭状态读取
    AutoRotationMode autorotationCapture();
    //pc或pad模式获取
    PcPadMode pcorpadmodeCapture();
    //旋转值获取
    RotationMode rotationCapture();

    //GstFlowReturn playnew_sample (GstElement *sink_elm, CustomData * data);
    //设置曝光模式
    RetStatus set_exposure_mode(ExposeMode exposure);
    //设置曝光值，单位是微秒（us），（手动曝光时有效）
    /*设置ev：默认自动曝光（AE）模式，可以设置EV以调节相对亮度。
    EV是一个偏移量，用户可以通过设置EV指定想要的曝光水平相对于当前AE结果的偏移。
    EV的范围和步进值可以在相机的XML配置文件（代码仓库中的<ipu6-camera-hal>/config/ubuntu/jsl/sensors/ov13858-uf.xml，
    客户端的/usr/share/default/etc/camera/sensors/ov13858-uf.xml）中修改
     <evRange value="-6,6"/>  <!—EV范围为-6~6的整数-->
     <evStep value="1,3"/>  <!—EV步进值，表示为有理数，为1/3-->
    其中EV范围的最大偏移量*步进值不大于3。上例中6*1/3=2，设置是有效的。
    set_exposure_ev 可接受的EV是evRange之内的整数。
    evStep是HAL内部需要的步进值，需要调整这个值使evRange× evStep <= 3，这是因为AE系统可以接受的实际偏移只有-3~+3，最终映射到AE算法内部的EV = API调用时传入的EV参数 × evStep。
    evRange和evStep的存在是为了使得用户可以更细粒度地控制EV值。以这里的设置为例，
    set_exposure_ev可接受的EV输入集合为{-6，-5，-4，-3，-2，-1，0，1，2，3，4，5，6}，映射到AE算法内部的值分别为{-2, -5/3, -4/3, -1, -2/3, -1/3, 0, 1/3, 2/3, 1, 4/3, 5/3, 2}。
    输入参数 ev 范围-5到5
    */
    RetStatus set_exposure_ev(int ev);
    //设置对焦模式
    RetStatus set_af_mode(AfMode afmode);
    //设置固定对焦距离（手动对焦时有效）,注意：有效值在（100-5000之间）
    RetStatus set_af_ff_distance(int mm_num);
    //停止设置ROI
    RetStatus stop_set_roi_region();
    //设置ROI坐标
    RetStatus set_roi_coordinate(int coordinate_x,int coordinate_y);
    enum FoldStatus
    {
        normal=1,      //水平向下，不改变pc/table模式，不改变前一个信号值
        upsidedown=2,  //
        left=3,        //
        right=4 ,      //
    };
    Q_ENUM(FoldStatus);
    //平板自带旋转传感器
    QOrientationSensor * qsensor;
    QTimer *time;
private:
    VideoDisplay *videoDisplay = nullptr;
    GstCamerasrc3A *g_cam3a;
    GstCamerasrc3AInterface *g_camera3a_interface;
    Mavis_Motor *mavis_motor;               //motor对象
    PngThread *pngthread;                   //png存储线程
    PngThread *pngthread1;                   //png存储线程
    PngThread *pngthread2;                   //png存储线程
    //PngThread *pngthread3;                   //png存储线程
    //PngThread *pngthread4;                   //png存储线程
    const char *previewresolution_c;        //预览分辨率
    const char *imageresolution_c;          //拍照分辨率
    const char *videoresolution_c;          //录像分辨率
    void getFrame(QImage frame);
    void get_previewresolution_c();         //将预览分辨率转换成const char*
    void get_imageresolution_c();           //将拍照分辨率转换成const char*
    void get_videoresolution_c();           //将录像分辨率转换成const char*
    //设置ROI
    RetStatus set_roi_region(int top,int bottom,int left,int right);
    //ROI坐标映射
    ret_status getRealMousePostion(int camId, int event_x, int event_y, int disWidth, int disHeight, int currentCamNum);
    //设置预览（录像）或拍照的分辨率
    void setting_resolution(void);
    //设置比例（分子，分母）
    bool setting_aspectratiocrop(const int numerator,const int denominator);
    //设置左右上下镜像
    void setting_mirroring(MirroringMode mirror);
    //设置旋转
    void setting_flip(FlipMode flipmode);
    //设置图片旋转
    void setting_image_flip(FlipMode flipmode);
    //设置video旋转
    void setting_video_flip(FlipMode flipmode);
    //在开始录像时，镜像或旋转判断函数
    void mirror_or_flip_func_video_start();
    //镜像或旋转判断函数
    void mirror_or_flip_func();
    //将自动旋转改变和PC模式改变的信号做一个统一的处理
    void autorotation_PCmode_DealChanged();
    //判断当前安卓应用有没有启用摄像头
    int android_if_open();
private Q_SLOTS:
    //自动旋转是否开启信号
    void autorotationChanged(bool res);
    //机器模式改变信号
    void PCmodeChanged(bool res);
    void rotationsChanged();    
    //定時器
    void TimerUpdate();
    void rece_sig_imagepath(QString str);
    void motor_sig_over(MotorReturn ret); //接收zyz自己写的motor运动结束槽函数
    void motor_slot_over_intel();               //接收intel提供的motor运动结束槽函数
Q_SIGNALS:
    void gst_sig_imagepath(QString str);
    void gst_motor_sig_over(MotorReturn ret);
    void gst_motor_sig_start_intel(bool);       //外部hw_motor按键发出的信号
    void gst_motor_sig_over_intel();
};

#endif // MAVIS_GST_H
